/*
 * complex_helper.hpp
 * Copyright (C) 2019 Stephan Seitz <stephan.seitz@fau.de>
 *
 * Distributed under terms of the GPLv3 license.
 */

#pragma once

extern "C++" {
#ifdef __CUDA_ARCH__
// TODO
#else
#include <complex>

template <class U, class V>
auto operator*(const std::complex<U> &complexNumber, const V &scalar)
    -> std::complex<U> {
  return std::complex<U>{std::real(complexNumber) * scalar,
                         std::imag(complexNumber) * scalar};
}

template <class U, class V>
auto operator*(const V &scalar, const std::complex<U> &complexNumber)
    -> std::complex<U> {
  return std::complex<U>{std::real(complexNumber) * scalar,
                         std::imag(complexNumber) * scalar};
}

template <class U, class V>
auto operator+(const std::complex<U> &complexNumber, const V &scalar)
    -> std::complex<U> {
  return std::complex<U>{std::real(complexNumber) + scalar,
                         std::imag(complexNumber)};
}

template <class U, class V>
auto operator+(const V &scalar, const std::complex<U> &complexNumber)
    -> std::complex<U> {
  return std::complex<U>{std::real(complexNumber) + scalar,
                         std::imag(complexNumber)};
}

template <class U, class V>
auto operator-(const std::complex<U> &complexNumber, const V &scalar)
    -> std::complex<U> {
  return std::complex<U>{std::real(complexNumber) - scalar,
                         std::imag(complexNumber)};
}

template <class U, class V>
auto operator-(const V &scalar, const std::complex<U> &complexNumber)
    -> std::complex<U> {
  return std::complex<U>{scalar - std::real(complexNumber),
                         std::imag(complexNumber)};
}

template <class U, class V>
auto operator/(const std::complex<U> &complexNumber, const V &scalar)
    -> std::complex<U> {
  return std::complex<U>{std::real(complexNumber) / scalar,
                         std::imag(complexNumber) / scalar};
}

template <class U, class V>
auto operator/(const V &scalar, const std::complex<U> &complexNumber)
    -> std::complex<U> {
  return std::complex<U>{scalar, 0} / complexNumber;
}

#endif
}
