# -*- coding: utf-8 -*-
#
# Copyright © 2019 Stephan Seitz <stephan.seitz@fau.de>
#
# Distributed under terms of the GPLv3 license.
"""

"""

import itertools

import pytest
import sympy
from sympy.functions import im, re

import pystencils
from pystencils import AssignmentCollection
from pystencils.data_types import TypedImaginaryUnit, TypedSymbol, create_type

X, Y = pystencils.fields('x, y: complex64[2d]')
A, B = pystencils.fields('a, b: float32[2d]')
S1, S2 = sympy.symbols('S1, S2')
# T64 = TypedSymbol('t', create_type('complex64'))
T64 = sympy.Symbol('t')

TEST_ASSIGNMENTS = [
    AssignmentCollection({X[0, 0]: 1j}),
    AssignmentCollection({
        S1: re(Y.center),
        S2: im(Y.center),
        X[0, 0]: 2j * S1 + S2
    }),
    AssignmentCollection({
        A.center: re(Y.center),
        B.center: im(Y.center),
    }),
    AssignmentCollection({
        Y.center: re(Y.center) + X.center + 2j,
    }),
    AssignmentCollection({
        T64: 2 + 4j,
        Y.center: X.center / T64,
    })
]

SCALAR_DTYPES = ['float32', 'float64']


@pytest.mark.parametrize("assignment, scalar_dtypes",
                         itertools.product(TEST_ASSIGNMENTS, SCALAR_DTYPES))
def test_complex_numbers(assignment, scalar_dtypes):
    ast = pystencils.create_kernel(assignment,
                                   target='cpu',
                                   data_type='float32')
    code = str(pystencils.show_code(ast))

    print(code)
    assert "Not supported" not in code

    kernel = ast.compile()
    assert kernel is not None


X, Y = pystencils.fields('x, y: complex128[2d]')
A, B = pystencils.fields('a, b: float64[2d]')
S1, S2 = sympy.symbols('S1, S2')
# T128 = pystencils.TypedSymbol('ts', create_type('complex128'))
T128 = sympy.Symbol('ts')

TEST_ASSIGNMENTS = [
    AssignmentCollection({X[0, 0]: 1j}),
    AssignmentCollection({
        S1: re(Y.center),
        S2: im(Y.center),
        X[0, 0]: 2j * S1 + S2
    }),
    AssignmentCollection({
        A.center: re(Y.center),
        B.center: im(Y.center),
    }),
    AssignmentCollection({
        Y.center: re(Y.center) + X.center + 2j,
    }),
    AssignmentCollection({
        T128: 2 + 4j,
        Y.center: X.center / T128,
    })
]

SCALAR_DTYPES = ['float32', 'float64']


@pytest.mark.parametrize("assignment, scalar_dtypes",
                         itertools.product(TEST_ASSIGNMENTS, SCALAR_DTYPES))
def test_complex_numbers_64(assignment, scalar_dtypes):
    ast = pystencils.create_kernel(assignment,
                                   target='cpu',
                                   data_type='double')
    code = str(pystencils.show_code(ast))

    print(code)
    assert "Not supported" not in code

    kernel = ast.compile()
    assert kernel is not None


def test_get_data_type():
    import numpy as np
    from pystencils.data_types import get_type_of_expression

    i = TypedImaginaryUnit(create_type('complex128'))
    assert get_type_of_expression(i+3).numpy_dtype == np.complex128
    assert get_type_of_expression(i+3.).numpy_dtype == np.complex128
    i = TypedImaginaryUnit(create_type('complex64'))
    assert get_type_of_expression(i+3, default_float_type='float32').numpy_dtype == np.complex64
    assert get_type_of_expression(i+3., default_float_type='float32').numpy_dtype == np.complex64
